def model_to_dict(obj, context):
    """
    Convert django model instance to dict
    """

    data = {}

    for field in obj._meta.fields:
        field_name = field.column if field.rel else field.name
        data[field.column] = getattr(obj, field_name)

    return data


def autodiscover(module_name='restapi'):
    from django.conf import settings

    try:
        from django.utils.module_loading import autodiscover_modules
    except ImportError:
        from django.utils.importlib import import_module
        from django.utils.module_loading import module_has_submodule
        autodiscover_modules = None

    if autodiscover_modules:
        autodiscover_modules(module_name)
    else:
        for app in settings.INSTALLED_APPS:
            mod = import_module(app)
            try:
                import_module('%s.%s' % (app, module_name))
            except:
                if module_has_submodule(mod, module_name):
                    raise



import functools
import logging
import warnings
from six.moves.urllib.parse import urlencode

from collections import OrderedDict, defaultdict

from .representations import (  # NOQA
        RepresentationAlreadyRegistered, ValidatorAlreadyRegistered,
        Representation, Validator, UnknownRepresentation,
        match_representation, NoRepresentationFound, NoMoreMediaTypes)
from .utils import (
        join_content_type_with_vnd,
        split_mediatype,
        get_types_to_check)
from .headers import parse_accept_header
from . import responses, urltemplate, serializers, deprecation


log = logging.getLogger(__name__)


def _join_ct_vnd(content_type, vnd):
    return join_content_type_with_vnd(content_type, vnd)


def dict_as_text(obj, ctx, depth=0):
    output = u''

    for key, value in obj.items():
        if isinstance(value, dict):
            value = u'\n'+dict_as_text(value, ctx, depth+1)
        output += u'%s%s: %s\n' % (' '*depth*2, key, value)

    return output


def resource_name_from_path(path):
    return urltemplate.remove_parameters(path).strip('/')


class Resource(object):
    def __init__(
            self, api, path, name=None,
            default_content_type='application/json',
            link_model=None, link_name=None):
        self._api = api
        self._path = (path or "").strip("/")
        self._required_parameters = urltemplate.get_parameters(self._path)
        self._callbacks = defaultdict(dict)
        self._registered_methods = set()
        self._name = name or resource_name_from_path(path)
        self._representations = OrderedDict()
        self._validators = OrderedDict()
        self._default_content_type = default_content_type
        self._supported_media_types = defaultdict(set)

        # register "pass-through" validators

        for content_type, serializer in serializers.get_all():
            self.add_validator(content_type=content_type)

        if link_model:
            self._api.register_view(
                    model=link_model, resource=self, view_name=link_name)

        # register aliases for the decorators
        for verb in ('GET', 'POST', 'PUT', 'DELETE', 'OPTIONS', 'PATCH'):
            setattr(
                self, verb.lower(), functools.partial(self._decorator, verb))

    def is_callback_registered(self, method, content_type=None):
        content_type = content_type or self._default_content_type
        return method in self._callbacks[content_type]

    def register_method_callback(self, callback, method, content_type=None):
        content_type = content_type or self._default_content_type
        self._callbacks[content_type][method] = callback
        self._registered_methods.add(method)
        self._supported_media_types[method].add(content_type)

    def get_method_supported_mediatypes(self, method):
        return list(self._supported_media_types[method])

    def get_callback(self, method, content_type=None):
        content_type = content_type or self._default_content_type
        return self._callbacks[content_type][method]

    def get_allowed_methods(self):
        return list(self._registered_methods)

    @property
    def default_content_type(self):
        return self._default_content_type

    def _decorator(self, method, accept=None):
        def wrapper(view):
            if self.is_callback_registered(method, content_type=accept):
                raise ValueError(
                        'Method `%s` is already registered' % method)
            self.register_method_callback(
                    view, method=method, content_type=accept)
            return view
        return wrapper

    def _match_representation(self, instance, ctx, accept=None):
        return match_representation(self, ctx, instance, accept=accept)

    def _http_response(self, response):
        content = ''
        content_type = self._default_content_type
        representation = None

        if response.data is not None:
            try:
                representation = self._match_representation(
                        response.data, response.context)
            except NoRepresentationFound:
                if isinstance(response, responses.SuccessfulResponse):
                    return self._http_response(
                        responses.NotAcceptableResponse(
                            response.context, headers=response.headers))
                elif isinstance(response, (
                        responses.ServerErrorResponse,
                        responses.ClientErrorResponse)):
                    # For errors use any representation supported by
                    # server. It is better to provide any information
                    # in any format instead of nothing.
                    # -- RFC7231 (Section 6.5 & 6.6)
                    #
                    # Restosaur will try to match most acceptible
                    # representation.

                    accept = response.context.headers.get('accept')
                    accepting = ['*/*;q=0.1']

                    if accept:
                        accept_parts = parse_accept_header(accept)
                        for part in accept_parts:
                            media_type, media_subtype = part[0].split('/')
                            accepting.insert(0, '%s/*;q=1' % media_type)

                    try:
                        representation = self._match_representation(
                                response.data, response.context,
                                accept=','.join(accepting))
                    except NoRepresentationFound:
                        # return no content and preserve status code
                        pass
                    else:
                        content = representation.transform(
                                    response.context, response.data)
                        content_type = _join_ct_vnd(
                               representation.content_type, representation.vnd)
                else:
                    # return no content and preserve status code
                    pass
            else:
                content = representation.transform(
                                response.context, response.data)
                content_type = _join_ct_vnd(
                       representation.content_type, representation.vnd)

        response.content_type = content_type
        response.representation = representation
        response.content = content
        response.serializer = (
                representation.serializer if representation else None)

        return response

    @property
    def name(self):
        return self._name

    @property
    def api(self):
        return self._api

    @property
    def path(self):
        return self._path

    @property
    def representations(self):
        result = []
        for models in self._representations.values():
            result += models.values()
        return result + self._api.representations

    def get_representations_for(self, model):
        result = []
        model_class = model
        for models in self._representations.values():
            matching_models = filter(
                    lambda x: x[0] == model_class or x[0] is None,
                    models.items())
            result += list(map(lambda x: x[1], matching_models))
        return result + self._api.get_representations_for(model)

    def has_representation_for(self, model, media_type):
        if media_type not in self._representations:
            return self._api.has_representation_for(model, media_type)

        types_to_check = get_types_to_check(model)

        has_resource_repr = any(map(
            lambda x: x in self._representations[media_type],
            types_to_check))

        return (has_resource_repr
                or self._api.has_representation_for(model, media_type))

    def get_representation(self, model, media_type):
        types_to_check = get_types_to_check(model)

        for cls in types_to_check:
            try:
                return self._representations[media_type][cls]
            except KeyError:
                pass
        try:
            return self._api.get_representation(model, media_type)
        except UnknownRepresentation:
            try:
                return self._representations[media_type][None]
            except KeyError:
                raise UnknownRepresentation(
                    'Neither %s nor %s have representation for %s and "%s"' % (
                        self, self._api, model, media_type))

    def link(self, model, name=None):
        """
        Create named link between model and resource
        """
        self._api.link(model=model, resource=self, name=name)

    def linked_model(self, name=None):
        """
        Decorator for linking `self` (the resource) with model
        """
        def register_model(model_class):
            self.link(model_class, name=name)
            return model_class
        return register_model

    def accept(self, media_type=None):
        media_type = media_type or self._default_content_type
        ct, vnd = split_mediatype(media_type)
        return self.validator(content_type=ct, vnd=vnd)

    def representation(self, model=None, media=None, serializer=None):
        def wrapped(func):
            if isinstance(media, (list, tuple)):
                content_types = map(split_mediatype, media)
            else:
                content_types = [split_mediatype(
                    media or self._default_content_type)]

            for ct, v, args in content_types:
                self.add_representation(
                    model=model, vnd=v, content_type=ct, qvalue=args.get('q'),
                    serializer=serializer, _transform_func=func)
            return func
        return wrapped

    def validator(self, vnd=None, content_type=None, serializer=None):
        def wrapped(func):
            self.add_validator(
                    vnd=vnd, content_type=content_type, serializer=serializer,
                    _validator_func=func)
            return func
        return wrapped

    def add_representation(
            self, model=None, vnd=None, content_type=None, qvalue=None,
            serializer=None, _transform_func=None):

        if model is None:
            warnings.warn(
                'Representation for resource %s should be narrowed to '
                'specified model. Starting from v0.9 representation will '
                'require model explicitely.' % self,
                deprecation.RemovedInRestosaur09Warning)
        content_type = content_type or self._default_content_type
        repr_key = _join_ct_vnd(content_type, vnd)

        if (repr_key in self._representations and
                not repr_key == self._default_content_type
                and model in self._representations[repr_key]):
            raise RepresentationAlreadyRegistered(
                    '%s: %s (%s)' % (self._path, repr_key, model))

        if model is None and qvalue is None:
            qvalue = 0.01
        elif qvalue is None:
            qvalue = self.api.get_default_qvalue(repr_key)

        obj = Representation(
                vnd=vnd, content_type=content_type, serializer=serializer,
                _transform_func=_transform_func, qvalue=qvalue)

        self._representations.setdefault(repr_key, {})
        self._representations[repr_key][model] = obj
        return obj

    def add_validator(
            self, vnd=None, content_type=None, serializer=None,
            _validator_func=None):

        content_type = content_type or self._default_content_type
        repr_key = _join_ct_vnd(content_type, vnd)

        if (repr_key in self._validators and
                not repr_key == self._default_content_type):
            raise ValidatorAlreadyRegistered(
                    '%s: %s' % (self._path, repr_key))

        obj = Validator(
                vnd=vnd, content_type=content_type, serializer=serializer,
                _validator_func=_validator_func)

        self._validators[content_type] = obj
        return obj

    def uri(self, context, params=None, query=None, append_query=False):
        assert params is None or isinstance(
                params, dict), "entity.uri() params should be passed as dict"

        params = params or {}

        uri = context.build_absolute_uri(self._path)
        uri = urltemplate.to_url(uri, params)

        final_query = {}

        if append_query:
            final_query.update(context.parameters)

        final_query.update(query or {})

        if final_query:
            uri += '?'+urlencode(final_query)

        return uri

    def __repr__(self):
        return '<Resource "%s">' % self.path



from restosaur.contrib.django import JsonAPI


api = JsonAPI('foo')
subapi = JsonAPI('bar')

baz = subapi.resource('/')


@baz.get()
def baz_view(ctx):
    return ctx.OK('baz here')


api.join(subapi)


urlpatterns = api.urlpatterns()



import json
import re
import unittest

from restosaur import API
from restosaur.contrib.django import API as DjangoAPI
from restosaur.contrib.django.dispatch import resource_dispatcher_factory

from .utils import response_content_as_text


def _urlpattern_regex(urlpattern):
    try:
        return urlpattern._regex  # django < 2.0
    except AttributeError:
        try:
            return urlpattern.pattern._regex  # django 2.0
        except AttributeError:
            return urlpattern.pattern.regex.pattern


class APITestCase(unittest.TestCase):
    def setUp(self):
        from django.test import RequestFactory

        super(APITestCase, self).setUp()
        self.rqfactory = RequestFactory()

    def call(self, api, resource, method, *args, **kw):
        rq = getattr(self.rqfactory, method)(resource.path, *args, **kw)
        return resource_dispatcher_factory(api, resource)(rq)


class APIPathsTestCase(APITestCase):
    def test_prepending_slash_to_api_path(self):
        api = API("foo")
        self.assertEqual(api.path, "/foo")

    def test_not_striping_slash_from_api_path(self):
        api = API("foo/")
        self.assertEqual(api.path, "/foo")

    def test_status_200OK_of_nonprefixed_api_path(self):
        api = API()
        root = api.resource("/")

        @root.get()
        def root_view(ctx):
            return ctx.Response({"root": "ok"})

        resp = self.call(api, root, "get")
        self.assertEqual(resp.status_code, 200)

    def test_valid_response_of_nonprefixed_api_path(self):
        api = API()
        root = api.resource("/")

        @root.get()
        def root_view(ctx):
            return ctx.Response({"root": "ok"})

        resp = self.call(api, root, "get")
        resp_json = json.loads(response_content_as_text(resp))
        self.assertEqual(resp_json["root"], "ok")



# coding: utf-8
from __future__ import unicode_literals

import datetime
import unittest

import pytz
import six
from restosaur import API, responses
from restosaur.contrib.django.dispatch import build_context


class ContextTestCase(unittest.TestCase):
    def setUp(self):
        from django.test import RequestFactory

        super(ContextTestCase, self).setUp()

        def create_context(method, path, resource, api=None, **kwargs):
            api = api or self.api
            request = getattr(self.rqfactory, method)(path)
            return api.make_context(
                host=request.get_host(),
                path=request.path,
                request=request,
                resource=resource,
                method=method,
                **kwargs
            )

        self.api = API("/")
        self.api2 = API("webapi/")
        self.api3 = API("webapi")
        self.rqfactory = RequestFactory()
        self.factory = create_context
        self.ctx = create_context("get", "", lambda ctx: None, api=self.api)
        self.ctx2 = create_context("get", "", lambda ctx: None, api=self.api2)
        self.ctx3 = create_context("get", "", lambda ctx: None, api=self.api3)

    def test_deserialized_property(self):
        ctx = self.factory("get", "/test/", lambda ctx: None, body="body")
        self.assertEqual(ctx.body, ctx.deserialized)


class TestContextBuilidURI(ContextTestCase):
    def test_using_current_location(self):
        ctx = self.factory("get", "/test/", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri(), "http://testserver/test/")

    def test_using_custom_location(self):
        ctx = self.factory("get", "/foo/", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri("/bar/"), "http://testserver/bar/")

    def test_appending_custom_parameter_to_current_location(self):
        ctx = self.factory("get", "/foo/", lambda ctx: None)
        self.assertEqual(
            ctx.build_absolute_uri(parameters={"bar": "baz"}),
            "http://testserver/foo/?bar=baz",
        )

    def test_appending_custom_parameter_to_custom_location(self):
        ctx = self.factory("get", "/foo/", lambda ctx: None)
        url = ctx.build_absolute_uri("/bar/", parameters={"spam": "eggs"})
        self.assertEqual(url, "http://testserver/bar/?spam=eggs")

    def test_overriding_parameter_on_current_location(self):
        ctx = self.factory("get", "/foo/?bar=spam", lambda ctx: None)
        url = ctx.build_absolute_uri(parameters={"bar": "eggs"})
        self.assertEqual(url, "http://testserver/foo/?bar=eggs")

    def test_overriding_parameter_on_custom_location(self):
        ctx = self.factory("get", "/foo/?bar=spam", lambda ctx: None)
        url = ctx.build_absolute_uri("/baz/", parameters={"bar": "eggs"})
        self.assertEqual(url, "http://testserver/baz/?bar=eggs")

    def test_encoding_parameter(self):
        ctx = self.factory("get", "/foo/", lambda ctx: None)
        url = ctx.build_absolute_uri(
            "/baz/", parameters={"bar": six.text_type("zażółć")}
        )
        self.assertEqual(url, "http://testserver/baz/?bar=za%C5%BC%C3%B3%C5%82%C4%87")

    def test_generating_absolute_uri_using_request_path(self):
        ctx = self.factory("get", "/", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri(), "http://testserver/")

    def test_generating_absolute_uri_using_slash_as_path(self):
        ctx = self.factory("get", "", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri("/"), "http://testserver/")

    def test_generating_absolute_uri_using_double_slash_as_path(self):
        ctx = self.factory("get", "", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri("//"), "http://testserver/")

    def test_generating_absolute_uri_using_triple_slash_as_path(self):
        ctx = self.factory("get", "", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri("///"), "http://testserver/")

    def test_generating_absolute_uri_using_path_with_one_preceding_slash(self):
        ctx = self.factory("get", "", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri("/some"), "http://testserver/some")

    def test_generating_absolute_uri_using_path_with_third_preceding_slashes(
        self,
    ):  # NOQA
        ctx = self.factory("get", "", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri("///some"), "http://testserver/some")

    def test_generating_absolute_uri_using_path_ending_with_slash(self):
        ctx = self.factory("get", "", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri("/some/"), "http://testserver/some/")

    def test_not_replacing_hostname_when_using_two_prepending_slashes_in_build_absolute_uri(
        self,
    ):  # NOQA
        ctx = self.factory("get", "", lambda ctx: None)
        self.assertEqual(ctx.build_absolute_uri("//some/"), "http://testserver/some/")

    def test_successful_generating_uri_using_shortcut_wrapper_to_resource(self):  # NOQA
        res = self.api.resource("foo/:pk")
        ctx = self.factory("get", "/bar/", lambda ctx: None)
        url = ctx.url_for(res, pk="test")
        self.assertEqual(url, "http://testserver/foo/test")

    def test_generating_uri_for_prefixed_api_and_slash_as_path(self):
        self.assertEqual(self.ctx2.build_absolute_uri("/"), "http://testserver/webapi/")

    def test_generating_uri_for_prefixed_api_and_path_wihout_preceding_slash(
        self,
    ):  # NOQA
        self.assertEqual(
            self.ctx2.build_absolute_uri("foo"), "http://testserver/webapi/foo"
        )

    def test_generating_uri_for_prefixed_api_and_path_wihout_preceding_slash_but_ending_with_slash(
        self,
    ):  # NOQA
        self.assertEqual(
            self.ctx2.build_absolute_uri("foo/"), "http://testserver/webapi/foo/"
        )

    def test_generating_uri_for_prefixed_api_without_slash_and_slash_as_path(
        self,
    ):  # NOQA
        self.assertEqual(self.ctx3.build_absolute_uri("/"), "http://testserver/webapi/")

    def test_generating_uri_for_prefixed_api_without_slash_and_path_wihout_preceding_slash(
        self,
    ):  # NOQA
        self.assertEqual(
            self.ctx3.build_absolute_uri("foo"), "http://testserver/webapi/foo"
        )

    def test_generating_uri_for_prefixed_api_without_slash_and_path_wihout_preceding_slash_but_ending_with_slash(
        self,
    ):  # NOQA
        self.assertEqual(
            self.ctx3.build_absolute_uri("foo/"), "http://testserver/webapi/foo/"
        )

    def test_successful_building_uri_with_multivalue_query_arg(self):
        ctx = self.factory("get", "", lambda ctx: None)
        uri = ctx.build_absolute_uri("/some", {"foo": ["bar", "baz"]})
        self.assertEqual(uri, "http://testserver/some?foo=bar&foo=baz")

    def test_successful_getting_mutlivalue_GET_args(self):
        res = self.api.resource("foo")
        rq = self.rqfactory.get("/foo?bar=baz&bar=qux")
        ctx = build_context(self.api, res, rq)
        self.assertTrue("baz" in ctx.parameters["bar"])
        self.assertTrue("qux" in ctx.parameters["bar"])


class TestContextIfModifiedSince(ContextTestCase):
    def test_succesful_parse_rfc_1123_example_time(self):
        from restosaur.context import parse_http_date

        dt = parse_http_date("foo", {"foo": "Tue, 15 Nov 1994 08:12:31 GMT"})
        self.assertEqual(
            dt, datetime.datetime(1994, 11, 15, 8, 12, 31, tzinfo=pytz.UTC)
        )

    def test_error_parse_rfc_1123(self):
        from restosaur.context import parse_http_date

        dt = parse_http_date("foo", {"foo": "invalid http date"})
        self.assertIsNone(dt)

    def test_that_date_is_modified(self):
        ctx = self.factory(
            "get",
            "/foo/",
            lambda ctx: None,
            headers={
                "if-modified-since": "Tue, 15 Nov 1994 08:12:31 GMT",
            },
        )
        modification_date = datetime.datetime(1995, 1, 1, 10, 0, 0, tzinfo=pytz.UTC)
        self.assertTrue(ctx.is_modified_since(modification_date))

    def test_that_date_is_modified_for_invalid_header(self):
        ctx = self.factory(
            "get",
            "/foo/",
            lambda ctx: None,
            headers={
                "if-modified-since": "invalid http header",
            },
        )
        modification_date = datetime.datetime(1995, 1, 1, 10, 0, 0, tzinfo=pytz.UTC)
        self.assertTrue(ctx.is_modified_since(modification_date))

    def test_that_date_is_modified_without_header(self):
        ctx = self.factory("get", "/foo/", lambda ctx: None)
        modification_date = datetime.datetime(1995, 1, 1, 10, 0, 0, tzinfo=pytz.UTC)
        self.assertTrue(ctx.is_modified_since(modification_date))

    def test_that_date_is_not_modified(self):
        ctx = self.factory(
            "get",
            "/foo/",
            lambda ctx: None,
            headers={
                "if-modified-since": "Tue, 15 Nov 1994 08:12:31 GMT",
            },
        )
        modification_date = datetime.datetime(1990, 1, 1, 10, 0, 0, tzinfo=pytz.UTC)
        self.assertFalse(ctx.is_modified_since(modification_date))


class TestContextResponseFactories(ContextTestCase):
    def setUp(self):
        super(TestContextResponseFactories, self).setUp()
        self.ctx = self.factory("get", "/foo/", lambda ctx: None)

    def test_base_response(self):
        obj = self.ctx.Response()
        self.assertTrue(isinstance(obj, responses.Response))

    def test_created_response(self):
        obj = self.ctx.Created()
        self.assertTrue(isinstance(obj, responses.CreatedResponse))

    def test_nocontent_response(self):
        obj = self.ctx.NoContent()
        self.assertTrue(isinstance(obj, responses.NoContentResponse))

    def test_unauthorized_response(self):
        obj = self.ctx.Unauthorized()
        self.assertTrue(isinstance(obj, responses.UnauthorizedResponse))

    def test_forbidden_response(self):
        obj = self.ctx.Forbidden()
        self.assertTrue(isinstance(obj, responses.ForbiddenResponse))

    def test_notfound_response(self):
        obj = self.ctx.NotFound()
        self.assertTrue(isinstance(obj, responses.NotFoundResponse))

    def test_methodnotallowed_response(self):
        obj = self.ctx.MethodNotAllowed()
        self.assertTrue(isinstance(obj, responses.MethodNotAllowedResponse))

    def test_notacceptable_response(self):
        obj = self.ctx.NotAcceptable()
        self.assertTrue(isinstance(obj, responses.NotAcceptableResponse))

    def test_entity_response(self):
        obj = self.ctx.Entity()
        self.assertTrue(isinstance(obj, responses.EntityResponse))

    def test_collection_response(self):
        obj = self.ctx.Collection([])
        self.assertTrue(isinstance(obj, responses.CollectionResponse))

    def test_validationerror_response(self):
        obj = self.ctx.ValidationError({})
        self.assertTrue(isinstance(obj, responses.ValidationErrorResponse))

    def test_notmodified_response(self):
        obj = self.ctx.NotModified()
        self.assertTrue(isinstance(obj, responses.NotModifiedResponse))

    def test_seeother_response(self):
        obj = self.ctx.SeeOther("/")
        self.assertTrue(isinstance(obj, responses.SeeOtherResponse))



import json

from django.test import Client, SimpleTestCase

from .utils import response_content_as_text


class URLPatternsTestCase(SimpleTestCase):
    def test_successful_calling_apiroot_without_path_prefix(self):
        c = Client()

        with self.settings(ROOT_URLCONF="tests.urls_root_without_prefix"):
            resp = c.get("/")
            self.assertEqual(resp.status_code, 200)

    def test_valid_response_from_apiroot_without_path_prefix(self):
        c = Client()

        with self.settings(ROOT_URLCONF="tests.urls_root_without_prefix"):
            resp = c.get("/")
            resp_json = json.loads(response_content_as_text(resp))
            self.assertEqual(resp_json["root"], "ok")

    def test_that_subresource_generates_valid_link_to_root_without_path_prefix(self):
        c = Client()

        with self.settings(
            ROOT_URLCONF="tests.urls_root_without_prefix",
        ):
            resp = c.get("/some/sub")
            resp_json = json.loads(response_content_as_text(resp))
            self.assertEqual(resp_json["root"], "http://testserver/")

    def test_successful_calling_apiroot_with_path_prefix(self):
        c = Client()

        with self.settings(ROOT_URLCONF="tests.urls_root_with_prefix"):
            resp = c.get("/api/")
            self.assertEqual(resp.status_code, 200)

    def test_successful_calling_some_resoutce_with_path_prefix(self):
        c = Client()

        with self.settings(ROOT_URLCONF="tests.urls_root_with_prefix"):
            resp = c.get("/api/some")
            resp_json = json.loads(response_content_as_text(resp))
            self.assertEqual(resp.status_code, 200)
            self.assertEqual(resp_json["some"], "ok")

    def test_successful_calling_some_subresoutce_with_path_prefix(self):
        c = Client()

        with self.settings(ROOT_URLCONF="tests.urls_root_with_prefix"):
            resp = c.get("/api/some/sub")
            resp_json = json.loads(response_content_as_text(resp))
            self.assertEqual(resp.status_code, 200)
            self.assertEqual(resp_json["some/sub"], "ok")

    def test_that_subresource_generates_valid_link_to_root_with_path_prefix(self):
        c = Client()

        with self.settings(
            ROOT_URLCONF="tests.urls_root_with_prefix",
        ):
            resp = c.get("/api/some/sub")
            resp_json = json.loads(response_content_as_text(resp))
            self.assertEqual(resp_json["root"], "http://testserver/api/")

    def test_that_api_trailing_prefix_does_not_affect_urlresolver(self):
        c = Client()

        with self.settings(ROOT_URLCONF="tests.urls_root_with_trailing_slash"):
            resp = c.get("/api/")
            self.assertEqual(resp.status_code, 200)

    def test_that_force_script_name_resolves_root_endpoint(self):
        c = Client()

        with self.settings(
            ROOT_URLCONF="tests.urls_root_force_script_name_prefix",
            FORCE_SCRIPT_NAME="/scriptname",
        ):
            resp = c.get("/api/")
            self.assertEqual(resp.status_code, 200)

    def test_that_force_script_name_resolves_subsome_endpoint(self):
        c = Client()

        with self.settings(
            ROOT_URLCONF="tests.urls_root_force_script_name_prefix",
            FORCE_SCRIPT_NAME="/scriptname",
        ):
            resp = c.get("/api/some/sub")
            self.assertEqual(resp.status_code, 200)

    def test_that_force_script_name_generates_valid_link_to_root(self):
        c = Client()

        with self.settings(
            ROOT_URLCONF="tests.urls_root_force_script_name_prefix",
            FORCE_SCRIPT_NAME="/scriptname",
        ):
            resp = c.get("/api/some/sub")
            resp_json = json.loads(response_content_as_text(resp))
            print(resp_json)
            self.assertEqual(resp_json["root"], "http://testserver/scriptname/api/")



import re
from collections import defaultdict

from .context import Context
from .linking import ModelLinksRegistry
from .representations import (ExceptionRepresentation, Representation,
                              RepresentationAlreadyRegistered,
                              UnknownRepresentation,
                              restosaur_exception_dict_as_dict,
                              restosaur_exception_dict_as_text)
from .resource import Resource
from .utils import get_types_to_check, join_content_type_with_vnd


class ModelViewAlreadyRegistered(Exception):
    pass


class ModelViewNotRegistered(Exception):
    pass


class BaseAPI(object):
    def __init__(
        self,
        path=None,
        middlewares=None,
        context_class=None,
        default_charset=None,
        debug=False,
        force_script_name=None,
    ):
        _path = (path or "").strip("/")
        self.path = "/%s" % _path if _path else ""
        self._mimetype_qvalues = {}
        self.debug = debug
        self.resources = []
        self.default_charset = default_charset or "utf-8"
        self.middlewares = middlewares or []
        self._representations = defaultdict(dict)  # type->repr_key
        self.context_class = context_class or Context
        self.model_links = ModelLinksRegistry()
        self.force_script_name = force_script_name or ""
        self.path_re = re.compile(
            r"^%s%s/"
            % (
                "/" + self.force_script_name.strip("/")
                if self.force_script_name
                else "",
                self.path,
            )
        )

    def make_context(self, **kwargs):
        return self.context_class(self, **kwargs)

    def add_resources(self, *resources):
        self.resources += resources

    def resource(self, path, *args, **kw):
        obj = Resource(self, path, *args, **kw)
        self.add_resources(obj)
        return obj

    def set_default_qvalue(self, mimetype, qvalue):
        self._mimetype_qvalues[mimetype] = qvalue

    def get_default_qvalue(self, mimetype):
        return self._mimetype_qvalues.get(mimetype)

    def add_representation(
        self,
        type_,
        content_type,
        vnd=None,
        qvalue=None,
        serializer=None,
        _transform_func=None,
    ):

        if type_ is None and qvalue is None:
            qvalue = 0.01
        elif qvalue is None:
            repr_key = join_content_type_with_vnd(content_type, vnd)
            qvalue = self.get_default_qvalue(repr_key)

        representation = Representation(
            content_type=content_type,
            vnd=vnd,
            serializer=serializer,
            _transform_func=_transform_func,
            qvalue=qvalue,
        )

        self.register_representation(type_, representation)

    def register_representation(self, type_, representation):

        content_type = representation.content_type
        vnd = representation.vnd
        repr_key = join_content_type_with_vnd(content_type, vnd)

        if (
            repr_key in self._representations
            and type_ in self._representations[repr_key]
        ):
            raise RepresentationAlreadyRegistered("%s for %s" % (repr_key, type_))

        self._representations[repr_key][type_] = representation

    def get_representation(self, model, media_type):
        if media_type not in self._representations:
            raise UnknownRepresentation(
                '%s has no representation for %s and "%s"' % (self, model, media_type)
            )

        types_to_check = get_types_to_check(model)

        for cls in types_to_check:
            try:
                return self._representations[media_type][cls]
            except KeyError:
                pass
        raise UnknownRepresentation(
            '%s has no representation for %s and "%s"' % (self, model, media_type)
        )

    def has_representation_for(self, model, media_type):
        if media_type not in self._representations:
            return False

        types_to_check = get_types_to_check(model)

        return any(
            map(lambda x: x in self._representations[media_type], types_to_check)
        )

    @property
    def representations(self):
        result = []
        for models in self._representations.values():
            result += models.values()
        return result

    def get_representations_for(self, model):
        result = []
        model_class = model
        for models in self._representations.values():
            matching_models = filter(
                lambda x: x[0] is model_class or x[0] is None, models.items()
            )
            result += list(map(lambda x: x[1], matching_models))

        return result

    def linked_resource(self, model, name=None):
        """
        Return linked resource for model instance or class.
        """
        return self.model_links.linked_resource(model=model, name=name)

    def linked_url(
        self, context, instance_or_class, name=None, parameters=None, query=None
    ):
        return self.model_links.url(
            context, instance_or_class, name=name, parameters=parameters, query=query
        )

    def link(self, model, resource, name=None):
        self.model_links.link(model=model, resource=resource, name=name)

    def model_for(self, resource, name=None):
        """
        A shortcut decorator for linking model between resource.

        The `resource` may be passed as a dotted string path
        to avoid circular import problems.
        """

        def register_link_for_model(model):
            self.link(model=model, resource=resource, name=name)
            return model

        return register_link_for_model


class API(BaseAPI):
    def __init__(self, *args, **kw):
        super(API, self).__init__(*args, **kw)

        # backward compatibility
        configure_plain_text_api(self)
        configure_json_api(self)


JSON = API


def configure_json_api(api):
    api.add_representation(
        ExceptionRepresentation,
        content_type="application/json",
        _transform_func=restosaur_exception_dict_as_dict,
        qvalue=0.9,
    )
    api.add_representation(dict, content_type="application/json", qvalue=0.1)

    # backward compatibility

    from .utils import Collection

    api.add_representation(Collection, content_type="application/json")


def configure_plain_text_api(api):
    api.add_representation(
        ExceptionRepresentation,
        content_type="text/plain",
        _transform_func=restosaur_exception_dict_as_text,
        qvalue=0.1,
    )


def api_factory(path=None, api_class=API, **kwargs):
    api = api_class(path, **kwargs)
    return api


def json_api_factory(path=None, api_class=API, **kwargs):
    api = api_factory(path=path, api_class=api_class, **kwargs)
    return api



import collections


class MultiValueDict(collections.MutableMapping):
    """
    QueryDict acts like a plain `dict` type, but it handles
    automatially multiple values for same key.

    The most safest representation of URI query parameters is a list
    of tuples, because the parameter names aren't unique. Unfortunately
    accessing list of tuples is not so handy, so a mapping is
    required.

    In most cases query parameters looks like a mapping of simple
    key => value pairs, so we're expecting just one value per key. But when
    value is a list, we're expecting that accessing a key will return that
    list, not last nor first value.

    The problematic case is for keys, for which we're expecting always a list
    of values, but just one was passed in URI. Accessing the key will give
    just straight value instead of expected list with one item. In that cases
    you should use `QueryDict.getlist()` directly, which returns always a list.

    The values are stored internally as lists.

    `.items()` method returns a list of (key, value) tuples, where value is
    a single value from a key's values list. This means that key may not be
    unique. This representation is compatible with `urllib.urlencode()`.

    `.keys()` returns unique key names, same as for pure `dict`.

    `.values()` returns list of same values, which can be accessed by key,

    `.lists()` returns internal representation as list of lists.
    """

    def __init__(self, initial=None):
        self._data = {}
        self.update(initial)

    def update(self, data):
        if data is None:
            return
        else:
            try:
                data = list(data.items())
            except AttributeError:
                pass
        keys = list(set([x[0] for x in data]))
        for key in keys:
            self._data[key] = []
        for key, value in data:
            if isinstance(value, (list, tuple)):
                for x in value:
                    self._data[key].append(x)
            else:
                self._data[key].append(value)

    def items(self):
        result = []
        for key, values in self._data.items():
            result += list(map(lambda x: (key, x), values))
        return result

    def getlist(self, key, default=None):
        return self._data.get(key, default)

    def lists(self):
        return self._data.items()

    def __setitem__(self, key, value):
        return self.update({key: value})

    def __getitem__(self, key):
        return self._data[key][-1]\
                if len(self._data[key]) < 2 else self._data[key]

    def __delitem__(self, key):
        del self._data[key]

    def __len__(self):
        return len(self._data)

    def __iter__(self):
        return iter(self._data)

    def __repr__(self):
        return repr(self._data)


QueryDict = MultiValueDict



import times2 as times
import warnings

from email.utils import formatdate

from .representations import ExceptionRepresentation
from .utils import Collection


NOTSET = 'NOTSET'


def dummy_converter(x, context):
    return x


def http_date(x):
    return formatdate(x, usegmt=True)


class StatusCodeMismatch(ValueError):
    def __init__(self, class_name, status_code):
        super(StatusCodeMismatch, self).__init__(
            '%s' % (self.class_name, self.status_code))


class BaseResponse(object):
    def __init__(
            self, context, data=None, status=200, headers=None,
            last_modified=None):

        self.headers = {}

        for key, value in (headers or {}).items():
            self[key] = value

        self.representation = None
        self.content_type = None
        self.context = context
        self._status = status
        self.data = data

        if last_modified:
            self.set_last_modified(last_modified)

    def set_last_modified(self, dt):
        if dt:
            self.headers['Last-Modified'] = http_date(times.to_unix(dt))
        else:
            self.headers.pop('Last-Modified', None)

    @property
    def status(self):
        return self._status

    @status.setter
    def set_status(self, status):
        self._validate_status_code(status)
        self._status = status

    def __setitem__(self, name, value):
        self.headers[name] = value

    def __getitem__(self, name):
        return self.headers[name]

    def _validate_status_code(self, status):
        pass


class InformationalResponse(BaseResponse):
    def _validate_status_code(self, status):
        if status < 100 or status >= 200:
            raise StatusCodeMismatch('Informational', status)


class SuccessfulResponse(BaseResponse):
    def _validate_status_code(self, status):
        if status < 200 or status >= 300:
            raise StatusCodeMismatch('Successful', status)


class RedirectionResponse(BaseResponse):
    def _validate_status_code(self, status):
        if status < 300 or status >= 400:
            raise StatusCodeMismatch('Redirection', status)


class ClientErrorResponse(BaseResponse):
    def _validate_status_code(self, status):
        if status < 400 or status >= 500:
            raise StatusCodeMismatch('ClientError', status)


class ServerErrorResponse(BaseResponse):
    def _validate_status_code(self, status):
        if status < 500 or status >= 600:
            raise StatusCodeMismatch('ServerError', status)


class ContinueResponse(InformationalResponse):
    def __init__(self, context, headers=None):
        super(ContinueResponse, self).__init__(
                context, status=100, headers=headers)


class OKResponse(SuccessfulResponse):
    def __init__(self, context, data=None, headers=None):
        super(OKResponse, self).__init__(
                context, data=data, status=200, headers=headers)


class CreatedResponse(SuccessfulResponse):
    def __init__(self, context, data=None, headers=None):
        super(CreatedResponse, self).__init__(
                context, data=data, status=201, headers=headers)


class AcceptedResponse(SuccessfulResponse):
    def __init__(self, context, data=None, headers=None):
        super(AcceptedResponse, self).__init__(
                context, data=data, status=202, headers=headers)


class NoContentResponse(SuccessfulResponse):
    def __init__(self, context, headers=None):
        # No content/data as stated in RFC7231 (6.3.5)
        super(NoContentResponse, self).__init__(
                context, status=204, headers=headers)


class ResetContentResponse(SuccessfulResponse):
    def __init__(self, context, headers=None):
        # No content/data as stated in RFC7231 (6.3.6)
        super(ResetContentResponse, self).__init__(
                context, status=205, headers=headers)


class MultipleChoicesResponse(RedirectionResponse):
    def __init__(self, context, data=None, headers=None):
        super(MultipleChoicesResponse, self).__init__(
                context, data=data, status=300, headers=headers)


class MovedPermanentlyResponse(RedirectionResponse):
    def __init__(self, context, url, data=None, headers=None):
        headers = headers or {}
        headers['Location'] = url
        super(MovedPermanentlyResponse, self).__init__(
                context, data=data, status=301, headers=headers)


class FoundResponse(RedirectionResponse):
    def __init__(self, context, data=None, headers=None):
        super(FoundResponse, self).__init__(
                context, data=data, status=302, headers=headers)


class SeeOtherResponse(RedirectionResponse):
    def __init__(self, context, url, data=None, headers=None):
        headers = headers or {}
        headers['Location'] = url
        super(SeeOtherResponse, self).__init__(
                context, data=data, status=303, headers=headers)


class NotModifiedResponse(RedirectionResponse):
    def __init__(self, context, data=None, headers=None):
        super(NotModifiedResponse, self).__init__(
                context, data=data, status=304, headers=headers)


class BadRequestResponse(ClientErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(BadRequestResponse, self).__init__(
                context, data=data, status=400, headers=headers)


class UnauthorizedResponse(ClientErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(UnauthorizedResponse, self).__init__(
                context, data=data, status=401, headers=headers)


class ForbiddenResponse(ClientErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(ForbiddenResponse, self).__init__(
                context, data=data, status=403, headers=headers)


class NotFoundResponse(ClientErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(NotFoundResponse, self).__init__(
                context, data=data, status=404, headers=headers)


class MethodNotAllowedResponse(ClientErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(MethodNotAllowedResponse, self).__init__(
                context, data=data, status=405, headers=headers)


class NotAcceptableResponse(ClientErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(NotAcceptableResponse, self).__init__(
                context, data=data, status=406, headers=headers)


class ConflictResponse(ClientErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(ConflictResponse, self).__init__(
                context, data=data, status=409, headers=headers)


class GoneResponse(ClientErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(GoneResponse, self).__init__(
                context, data=data, status=410, headers=headers)


class UnsupportedMediaTypeResponse(ClientErrorResponse):
    def __init__(self, context, headers=None):
        super(UnsupportedMediaTypeResponse, self).__init__(
                context, data=None, status=415, headers=headers)


class InternalErrorResponse(ServerErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(InternalErrorResponse, self).__init__(
                context, data=data, status=500, headers=headers)


class NotImplementedResponse(ServerErrorResponse):
    def __init__(self, context, data=None, headers=None):
        super(NotImplementedResponse, self).__init__(
                context, data=data, status=501, headers=headers)


# deprecated custom responses

class CollectionResponse(SuccessfulResponse):
    def __init__(self, context, iterable, totalCount=None, key=None, **kwargs):
        warnings.warn(
            '`CollectionResponse` will be removed in Restosaur v0.9. '
            'You should use plain `Response` or `OKResponse` '
            '(ctx.Response() / ctx.OK() respectively).',
            DeprecationWarning, stacklevel=3)

        coll_obj = Collection(
                context, iterable, key=key, totalcount=totalCount)
        super(CollectionResponse, self).__init__(
                context, data=coll_obj, **kwargs)


class EntityResponse(OKResponse):
    pass


Response = OKResponse


class ValidationErrorResponse(ClientErrorResponse):
    def __init__(self, context, errors, headers=None):
        resp = {
                'errors': errors,
                }
        super(ValidationErrorResponse, self).__init__(
                context, data=resp, status=422, headers=headers)


def exception_response_factory(context, ex, tb=None, extra=None, cls=None):
    if not cls:
        if isinstance(ex, NotImplementedError):
            cls = NotImplementedResponse
        else:
            cls = InternalErrorResponse

    data = ExceptionRepresentation(ex, tb=tb, extra=extra)
    response = cls(context=context, data=data)
    data.status_code = response.status

    return response



import flask
import functools

from restosaur.headers import normalize_header_name
from restosaur.urltemplate import RE_PARAMS
from restosaur.dispatch import resource_dispatcher_factory


def to_flask_pattern(path):
    return RE_PARAMS.sub('/<\\2>', path)


def build_context(api, resource, request):
    is_secure = request.url.startswith('https')
    headers = dict(map(
            lambda x: (normalize_header_name(x[0]), x[1]),
            request.headers.items()))

    return api.make_context(
            host=request.host, path=request.path,
            method=request.method, parameters=request.args,
            data=request.data, files=request.files, raw=request.get_data(),
            charset=request.charset or api.default_charset,
            secure=is_secure, encoding=request.charset,
            resource=resource, request=request, headers=headers,
            content_type=request.content_type,
            content_length=request.content_length)


def build_http_response(app, response, resource):
    if response is None:
        return flask.Response()

    if isinstance(response, flask.Response):
        return response

    response = resource._http_response(response)

    if response.serializer:
        content = response.serializer.dumps(response.content)
    else:
        content = ''

        http_resp = app.make_response(
                (content, response.status, response.headers))
        http_resp.headers['Content-Type'] = response.content_type
        return http_resp


def register_api(api, app):
    """
    Bind the `api` instance to the Flask `app` application
    """

    def response_builder(response, resource):
        return build_http_response(app, response, resource)

    api.debug = app.config['DEBUG']

    for resource in api.resources:
        path = api.path+to_flask_pattern(resource._path)

        if not path.startswith('/'):
            path = '/'+path

        dispatcher = resource_dispatcher_factory(
            api, resource, response_builder, context_builder=build_context)

        endpoint = 'restosaour_resource_%s' % id(resource)

        app.add_url_rule(
                path, endpoint=endpoint,
                view_func=functools.partial(dispatcher, flask.request),
                methods=resource.get_allowed_methods())



import six

from . import contentnegotiation


class Collection(object):
    def __init__(
            self, context, iterable, key=None, totalcount=None,
            totalcount_key=None):

        self.totalcount = totalcount or len(iterable)
        self.iterable = iterable
        self.totalcount_key = totalcount_key or 'totalCount'
        self.key = key or 'items'


def join_content_type_with_vnd(content_type, vnd):
    if not vnd:
        return content_type

    x, y = content_type.split('/')
    return '%s/%s+%s' % (x, vnd, y)


def split_mediatype(mt):
    """
    Split media type into (content_type, vnd, params) tuple

    Function converts `type/[tree.]subtype[+suffix][;params]`
    into generic mediatype together with subtype without suffix.
    Extra parameteres are returned as third element of the tuple.
    """

    type_, subtype, args = contentnegotiation.parse_media_type(mt)
    suffix = subtype.split('+')[-1] if '+' in subtype else None

    if suffix and suffix not in (
            'json', 'zip', 'ber', 'der', 'fastinfoset', 'wbxml'):
        raise ValueError('Suffix "%s" is not supported (see: RFC6839)')

    ct = '%s/%s' % (type_, suffix or subtype)
    vnd = subtype.split('+')[0] if suffix else None
    return ct, vnd, args


def generic_mediatype(mt):
    return split_mediatype(mt)[0]


def force_bytes(s, encoding='utf-8', errors='strict'):
    if isinstance(s, bytes):
        if encoding == 'utf-8':
            return s
        else:
            return s.decode('utf-8', errors).encode(encoding, errors)
    if isinstance(s, memoryview):
        return bytes(s)
    try:
        return s.encode(encoding, errors)
    except AttributeError:
        return str(s).encode(encoding, errors)


def force_text(s, encoding='utf-8', errors='strict'):
    if issubclass(type(s), six.text_type):
        return s
    try:
        if not issubclass(type(s), six.string_types):
            if six.PY3:
                if isinstance(s, bytes):
                    s = six.text_type(s, encoding, errors)
                else:
                    s = six.text_type(s)
            elif hasattr(s, '__unicode__'):
                s = six.text_type(s)
            else:
                s = six.text_type(bytes(s), encoding, errors)
        else:
            s = s.decode(encoding, errors)
    except UnicodeDecodeError as e:
        if not isinstance(s, Exception):
            raise
        else:
            # If we get to here, the caller has passed in an Exception
            # subclass populated with non-ASCII bytestring data without a
            # working unicode method. Try to handle this without raising a
            # further exception by individually forcing the exception args
            # to unicode.
            s = ' '.join(force_text(arg, encoding, errors) for arg in s)
    return s


def get_types_to_check(cls):
    if cls is None:
        return [None]
    else:
        return type.mro(cls)



from django.apps import AppConfig


class RestosaurAppConfig(AppConfig):
    name = 'restosaur'
    verbose_name = 'Restosaur'

    def ready(self):
        from .settings import (
                AUTODISCOVER_MODULE, AUTODISCOVER_MODULES, AUTODISCOVER)
        from .utils import autodiscover

        if AUTODISCOVER:
            modules = list(AUTODISCOVER_MODULES)

            if AUTODISCOVER_MODULE:
                modules.append(AUTODISCOVER_MODULE)

            autodiscover(*modules)



import traceback

from .utils import Collection, join_content_type_with_vnd, force_text
from . import serializers, contentnegotiation


class RepresentationAlreadyRegistered(Exception):
    pass


class UnknownRepresentation(Exception):
    pass


class ValidatorAlreadyRegistered(Exception):
    pass


class NoMoreMediaTypes(Exception):
    pass


class NoRepresentationFound(Exception):
    pass


def _pass_through_trasnform(x, ctx):
    return x


def _pass_through_validation(x, ctx):
    return x


class Representation(object):
    def __init__(
            self, vnd=None, content_type='application/json', qvalue=None,
            serializer=None, _transform_func=None):

        self.qvalue = qvalue if qvalue is not None else 1
        self.serializer = serializer or serializers.get(content_type)
        self.content_type = content_type
        self.vnd = vnd
        self._transform_func = _transform_func or _pass_through_trasnform

    def transform(self, context, obj):
        """
        Renders representation of `obj` as raw content
        """
        if isinstance(obj, Collection):
            items = map(
                lambda x: self._transform_func(x, context), obj.iterable)
            data = {
                    obj.key: list(items),
                    obj.totalcount_key: len(obj.iterable),
                    }
        else:
            data = self._transform_func(obj, context)
        return data

    def serialize(self, context, data):
        return self.serializer.dumps(context, data)

    def render(self, context, obj):
        return self.serialize(context, self.transform(context, obj))

    def media_type(self):
        mt = join_content_type_with_vnd(self.content_type, self.vnd)
        return '%s; q=%s' % (mt, self.qvalue)


class Validator(object):
    def __init__(
            self, vnd=None, content_type='application/json',
            serializer=None, _validator_func=None):

        self.serializer = serializer or serializers.get(content_type)
        self.content_type = content_type
        self.vnd = vnd
        self._validator_func = _validator_func or _pass_through_validation

    def parse(self, context):
        """
        Parses raw representation content and builds object
        """
        return self._validator_func(self.serializer.loads(context), context)


class ExceptionRepresentation(object):
    def __init__(self, ex, tb=None, extra=None):
        super(ExceptionRepresentation, self).__init__()
        self.exc_type = type(ex)
        self.exc_value = ex
        self.tb = tb
        self.extra = extra
        self.status_code = 500


def restosaur_exception_dict_as_text(obj, ctx):
    output = u'\n'.join(
            traceback.format_exception(obj.exc_type, obj.exc_value, obj.tb))
    return output


def restosaur_exception_dict_as_dict(obj, ctx):
    data = {}
    data.update(obj.extra or {})
    data.update({
            'class': str(obj.exc_type),
            'error': force_text(obj.exc_value),
            })

    if obj.tb:
        def stack_trace(x):
            return dict(zip(['file', 'line', 'fn', 'source'], x))
        data['traceback'] = list(
                map(stack_trace, traceback.extract_tb(obj.tb)))

    return data


def match_representation(resource, ctx, instance, accept=None):
    """
    Match representation `resource` (or resource-like) representation
    of `instance`, based on `ctx`. Representations matching may be
    narrowed by `accept` media type.
    """

    # Use "*/*" as default -- RFC 7231 (Section 5.3.2)
    accept = accept or ctx.headers.get('accept') or '*/*'

    exclude = []

    if instance is None:
        model = None
    else:
        model = type(instance)

    representations = resource.get_representations_for(model)

    while True:
        try:
            mediatype = _match_media_type(
                    accept, representations, exclude=exclude)
        except NoMoreMediaTypes:
            break

        if not resource.has_representation_for(model, mediatype):
            if resource.has_representation_for(None, mediatype):
                return resource.get_representation(None, mediatype)
            exclude.append(mediatype)
        else:
            return resource.get_representation(model, mediatype)

    raise NoRepresentationFound(
        '%s has no registered representation handler for `%s`' % (
            model, accept))


def _match_media_type(accept, representations, exclude=None):
    exclude = exclude or []

    def _drop_mt_args(x):
        return x.split(';')[0]

    mediatypes = list(set(filter(
            lambda x: _drop_mt_args(x) not in exclude, map(
                lambda x: x.media_type(), representations))))

    if not mediatypes:
        raise NoMoreMediaTypes

    mediatype = contentnegotiation.best_match(mediatypes, accept)

    if not mediatype:
        raise NoMoreMediaTypes

    return _drop_mt_args(mediatype)



import warnings

from .contrib.django.apps import *  # NOQA
from .deprecation import RemovedInRestosaur08Warning

warnings.warn(
        '`%s` module is deprecated and will be removed in v0.8.'
        'Please replace `restosaur` in INSTALLED_APPS using '
        '``restosaur.contrib.django`` name' % __name__,
        RemovedInRestosaur08Warning, stacklevel=2)



import logging
import sys

from .exceptions import Http404
from . import responses, serializers, contentnegotiation


log = logging.getLogger(__name__)


class DefaultResourceDispatcher(object):
    def __init__(self, resource):
        self.resource = resource

    def dispatch(self, ctx, args=None, kwargs=None):
        resource = self.resource
        method = ctx.method

        # support for X-HTTP-METHOD-OVERRIDE

        method = ctx.headers.get('x-http-method-override') or method

        # Check request method and raise MethodNotAllowed if unsupported

        allowed_methods = resource.get_allowed_methods()

        if method not in allowed_methods:
            headers = {
                    'Allow': ', '.join(allowed_methods),
                }
            return ctx.MethodNotAllowed({
                'error': 'Method `%s` is not registered for resource `%s`' % (
                    method, resource._path)}, headers=headers)

        # Negotiate payload content type and store the best matching
        # result in ctx.request_content_type

        if ctx.content_type and ctx.content_length:
            media_types = resource.get_method_supported_mediatypes(method)
            if media_types:
                ctx.request_content_type = contentnegotiation.best_match(
                        media_types, ctx.content_type)
            else:
                # server does not support any representation
                ctx.request_content_type = None
        elif ctx.content_length:
            # No payload content-type was provided.
            # According to RFC7231 (Section 3.1.1.5) server may assume
            # "application/octet-stream" or try to examine the type.

            ctx.request_content_type = 'application/octet-stream'
        else:
            # No payload
            ctx.request_content_type = None

        # match response representation, serializer and content type

        if ctx.content_length and ctx.content_type:
            if ctx.request_content_type:
                if ctx.raw:
                    try:
                        ctx.validator = resource._validators[
                                    ctx.request_content_type]
                    except KeyError:
                        pass
                    else:
                        try:
                            ctx.body = ctx.validator.parse(ctx)
                        except serializers.DeserializationError as ex:
                            resp = responses.exception_response_factory(
                                    ctx, ex, cls=responses.BadRequestResponse)
                            return resp
            elif not ctx.content_length:
                ctx.body = None
            else:
                return ctx.UnsupportedMediaType()

        log.debug('Calling %s, %s, %s' % (method, args, kwargs))

        try:
            callback = resource.get_callback(method, ctx.request_content_type)
            resp = self.do_call(callback, ctx, args=args, kwargs=kwargs)
        except Http404:
            return ctx.NotFound()
        except Exception as ex:
            if resource._api.debug:
                tb = sys.exc_info()[2]
            else:
                tb = None
            resp = responses.exception_response_factory(ctx, ex, tb)
            log.exception(
                    'Internal Server Error: %s', ctx.request.path,
                    exc_info=sys.exc_info(),
                    extra={
                        'status_code': resp.status,
                        'context': ctx,
                    }
            )
            return resp
        else:
            if not resp:
                raise TypeError(
                        'Function `%s` does not return '
                        'a response object' % callback)
            return resp

    def do_call(self, callback, ctx, args=None, kwargs=None):
        return callback(ctx, *args, **kwargs)


def resource_dispatcher_factory(
        api, resource, response_builder, context_builder,
        dispatcher_class=DefaultResourceDispatcher):

    dispatcher = dispatcher_class(resource)

    def dispatch_request(request, *args, **kw):
        ctx = context_builder(api, resource, request)
        bypass_processing = False
        middlewares_called = []

        for middleware in api.middlewares:
            middlewares_called.append(middleware)

            try:
                method = middleware.process_request
            except AttributeError:
                pass
            else:
                response = method(request, ctx)
                if response:
                    bypass_processing = True
                    break

        if not bypass_processing:
            response = dispatcher.dispatch(ctx, args=args, kwargs=kw)

        response = response or None

        for middleware in reversed(api.middlewares):
            try:
                method = middleware.process_response
            except AttributeError:
                pass
            else:
                new_response = method(request, response, ctx)
                if new_response:
                    response = new_response

        return response_builder(response, resource)
    return dispatch_request



from .. import urltemplate


class ResourceAlreadyRegistered(Exception):
    pass


def autogenerate_resource_name(resource):
    path = urltemplate.remove_parameters(resource.path)

    try:
        name = list(filter(None, path.split('/')))[-1]
    except IndexError:
        name = '/'

    return name


def always_true(context):
    return True


class ApiRoot(object):
    def __init__(self, root_resource=None):
        """
        Creates instance of ApiRoot registry.

        If `root_resource` is provided, the ApiRoot`s view will be registered
        as a root_resource`s service for HTTP GET method.
        """

        self.resources = {}

        if root_resource:
            root_resource.get()(self.as_view())
            root_resource.add_representation(
                    model=type(self), content_type='application/json',
                    _transform_func=type(self).as_dict)

    def register(self, resource, name=None, condition=None):
        name = name or autogenerate_resource_name(resource)

        if name in self.resources:
            raise ResourceAlreadyRegistered(name)
        condition = condition or always_true
        self.resources[name] = {'resource': resource, 'condition': condition}

    def expose(self, name, condition=None):
        def wrap(resource):
            self.register(resource, name, condition=condition)
            return resource
        return wrap

    def as_dict(self, ctx):
        data = {}

        for name, meta in self.resources.items():
            if meta['condition'](ctx):
                data[name] = meta['resource'].uri(ctx)

        return data

    def as_view(self):
        def get_api_root(ctx):
            return ctx.OK(self)
        return get_api_root



import email
import functools

import six
import times2 as times

from .datastructures import QueryDict

try:
    import urlparse
except ImportError:
    from urllib import parse as urlparse

try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode

from . import responses
from .loading import load_resource
from .representations import match_representation
from .utils import force_bytes


def parse_http_date(header, headers):
    if header in headers and headers[header]:
        timetuple = email.utils.parsedate_tz(headers[header])
        try:
            return times.from_unix(email.utils.mktime_tz(timetuple))
        except (TypeError, ValueError):
            pass


class Context(object):
    def __init__(
        self,
        api,
        host="localhost",
        path="/",
        method="GET",
        parameters=None,
        body=None,
        data=None,
        files=None,
        raw=None,
        extra=None,
        headers=None,
        charset=None,
        secure=False,
        encoding="utf-8",
        resource=None,
        request=None,
        content_length=None,
        content_type=None,
    ):
        self.method = method
        self.api = api
        self.charset = charset
        self.headers = headers or {}
        self.resource = resource
        self.request = request
        self.encoding = encoding
        self.secure = secure
        self.host = (host or "").strip("/")
        self.path = self.api.path_re.sub("", (path or ""), count=1).lstrip("/")
        print(path, self.path, self.api.path_re)
        self.body = body
        self.raw = raw
        self.parameters = QueryDict(parameters)  # GET
        self.data = data or {}  # POST
        self.files = files or {}  # FILES
        self.deserializer = None
        self.content_type = content_type
        self.content_length = content_length
        self.extra = extra or {}

    def build_absolute_uri(self, path=None, parameters=None):
        """
        Returns absolute uri to the specified `path` with optional
        query string `parameters`.

        If no `path` is provided, the current request full path
        (including query string) will be used and extended by
        optional `parameters`.
        """
        path = ((path or "") if path is not None else self.path).lstrip("/")

        def build_uri(path):
            return "http%s://%s%s%s/%s" % (
                "s" if self.secure else "",
                self.host,
                self.api.force_script_name,
                self.api.path,
                path,
            )

        params = QueryDict()
        uri = build_uri(path)
        if not path:
            params.update(self.parameters.items())

        # todo: change to internal restosaur settings
        enc = self.encoding

        params.update(parameters or {})
        params = list(map(lambda x: (x[0], force_bytes(x[1], enc)), params.items()))

        if params:
            return "%s?%s" % (uri, urlencode(params))
        else:
            return uri

    def match_representation(self, model):
        return match_representation(self.resource, self, model)

    def transform_representation(self, model):
        representation = self.match_representation(model)
        return representation._transform_func(model, self)

    def url(self, model=None, resource=None, name=None, parameters=None, query=None):
        """
        Create URL for model named link or resource
        with optional query parameters
        """

        if model and resource:
            raise ValueError("Provide `model` or `resource`. Both set.")

        if model:
            return self.model_url(model, name=name, parameters=parameters, query=query)

        if name:
            raise ValueError("Named link must be used with model")

        if not resource:
            raise ValueError("Resource or model must is required")

        return self.resource_url(resource, parameters=parameters, query=query)

    def self_url(self, query=None, append_query=False):
        """
        Create URL pointing to self with optional query parameters.

        If `append_query` is True, the current query string parameters
        will be added.
        """

        return self.resource.uri(self, query=query, append_query=append_query)

    def url_for(self, resource, **kwargs):
        """
        Deprecated resource URL generator
        """
        return self.resource_url(resource, parameters=kwargs)

    def model_url(self, model, name=None, query=None, parameters=None):
        """
        Generate URL for model's named link with optional query parameters.
        Model instance or class is accepted.

        For classes you may use `parameters` argument to provide values
        for path template.
        """
        return self.api.linked_url(
            self, model, name=name, parameters=parameters, query=query
        )

    def resource_url(self, resource, parameters=None, query=None):
        """
        Generate URL for the resource using path parameters
        and optional query string parameters.
        """

        if isinstance(resource, six.string_types):
            resource = load_resource(resource)
        return resource.uri(self, params=parameters, query=query)

    def is_modified_since(self, dt):
        """
        Compares datetime `dt` with `If-Modified-Since` header value.
        Returns True if `dt` is newer than `If-Modified-Since`,
        False otherwise.
        """
        if_modified_since = parse_http_date("if-modified-since", self.headers)

        dt = times.make_aware(dt, tz="UTC")  # assume tz=utc if not set

        if if_modified_since:
            return (times.to_unix(dt.replace(microsecond=0))) > (
                times.to_unix(if_modified_since)
            )

        return True

    @property
    def deserialized(self):
        return self.body

    def wrap(self, func):
        """Wrap `func` with the Context instance as a last argument"""

        @functools.wraps(func)
        def wrapped(*args, **kwargs):
            args = list(args) + [self]
            return func(*args, **kwargs)

        return wrapped

    # response factories

    def Continue(self, *args, **kwargs):  # 100
        return responses.ContinueResponse(self, *args, **kwargs)

    def OK(self, *args, **kwargs):  # 200
        return responses.OKResponse(self, *args, **kwargs)

    def Response(self, *args, **kwargs):  # deprecated 200-like response
        return self.OK(*args, **kwargs)

    def Created(self, *args, **kwargs):  # 201
        return responses.CreatedResponse(self, *args, **kwargs)

    def Accepted(self, *args, **kwargs):  # 202
        return responses.AcceptedResponse(self, *args, **kwargs)

    def NoContent(self, *args, **kwargs):  # 204
        return responses.NoContentResponse(self, *args, **kwargs)

    def MovedPermanently(self, *args, **kwargs):  # 301
        return responses.MovedPermanentlyResponse(self, *args, **kwargs)

    def Found(self, *args, **kwargs):  # 302
        return responses.FoundResponse(self, *args, **kwargs)

    def SeeOther(self, *args, **kwargs):  # 303
        return responses.SeeOtherResponse(self, *args, **kwargs)

    def NotModified(self, *args, **kwargs):  # 304
        return responses.NotModifiedResponse(self, *args, **kwargs)

    def BadRequest(self, *args, **kwargs):  # 400
        return responses.BadRequestResponse(self, *args, **kwargs)

    def Unauthorized(self, *args, **kwargs):  # 401
        return responses.UnauthorizedResponse(self, *args, **kwargs)

    def Forbidden(self, *args, **kwargs):  # 403
        return responses.ForbiddenResponse(self, *args, **kwargs)

    def NotFound(self, *args, **kwargs):  # 404
        return responses.NotFoundResponse(self, *args, **kwargs)

    def MethodNotAllowed(self, *args, **kwargs):  # 405
        return responses.MethodNotAllowedResponse(self, *args, **kwargs)

    def NotAcceptable(self, *args, **kwargs):  # 406
        return responses.NotAcceptableResponse(self, *args, **kwargs)

    def Conflict(self, *args, **kwargs):  # 409
        return responses.ConflictResponse(self, *args, **kwargs)

    def Gone(self, *args, **kwargs):  # 410
        return responses.GoneResponse(self, *args, **kwargs)

    def UnsupportedMediaType(self, *args, **kwargs):  # 415
        return responses.UnsupportedMediaTypeResponse(self, *args, **kwargs)

    def ValidationError(self, *args, **kwargs):  # 422 WEBDAV Deprecated
        return responses.ValidationErrorResponse(self, *args, **kwargs)

    def InternalServerError(self, *args, **kwargs):
        return responses.InternalErrorResponse(self, *args, **kwargs)

    InternalError = InternalServerError

    def Entity(self, *args, **kwargs):  # deprecated, 200
        return responses.EntityResponse(self, *args, **kwargs)

    def Collection(self, *args, **kwargs):  # deprecated, 200
        return responses.CollectionResponse(self, *args, **kwargs)
